from os import environ
import logging
from pathlib import Path
import yaml

from sqlalchemy.ext.asyncio.engine import AsyncEngine
from sqlalchemy.orm.session import sessionmaker

logger = logging.getLogger(__name__)
ENV = environ.get('env', 'prod')


class Config:
    app: dict
    postgres: dict
    storage: dict
    map: dict
    security: dict
    version: str
    engine: AsyncEngine
    session_maker: sessionmaker

    def __init__(self) -> None:
        from ._version import __version__
        self.version = __version__


conf = Config()


def set_app_config(app) -> None:
    raw_configs = []
    with open(Path(__file__).parent / 'defaults.yml') as cf:
        raw_configs.append(cf.read())
    for cf_path in (
        Path(Path.cwd().root) / 'etc' / 'gisaf' / ENV,
        Path.home() / '.local' / 'gisaf' / ENV
    ):
        try:
            with open(cf_path.with_suffix('.yml')) as cf:
                raw_configs.append(cf.read())
        except FileNotFoundError:
            pass

    yaml_config = yaml.safe_load('\n'.join(raw_configs))

    conf.app = yaml_config['app']
    conf.postgres = yaml_config['postgres']
    conf.storage = yaml_config['storage']
    conf.map = yaml_config['map']
    conf.security = yaml_config['security']
    # create_dirs()


# def create_dirs():
#     """
#     Create the directories needed for a proper functioning of the app
#     """
#     ## Avoid circular imports
#     from treetrail.api_v1 import attachment_types
#     for type in attachment_types:
#         base_dir = Path(conf.storage['root_attachment_path']) / type
#         base_dir.mkdir(parents=True, exist_ok=True)
#     logger.info(f'Cache dir: {get_cache_dir()}')
#     get_cache_dir().mkdir(parents=True, exist_ok=True)


def get_cache_dir() -> Path:
    return Path(conf.storage['root_cache_path'])